/* $Id: kgi_commands.h,v 1.14 2000/06/20 23:48:58 becka Exp $
******************************************************************************

   KGIcon - command definitions

   Copyright (C) 2000	Andreas Beck	[becka@ggi-project.org]
   Copyright (C) 1997	Steffen Seeger	[seeger@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _KGI_KGI_COMMANDS_H
#define _KGI_KGI_COMMANDS_H

#include <kgi/types.h>

/* 
 * 	The ioctl's
 * 
 * 	Bit coding is according to <asm/ioctl.h> now. The <type>
 * 	gives the subsystem and the <nr> the command.
 * 
 *  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *  !!!	NOTE: DO NOT CHANGE WITHOUT CHANGING THE KERNEL DRIVER INTERFACE !!!
 *  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 */ 


#define KGI_IOCTLBASE	0x00

#define IOC_DRIVER	(KGI_IOCTLBASE+0x00)	/* kernel driver	*/
#define IOC_MONITOR	(KGI_IOCTLBASE+0x01)	/* monitor driver	*/
#define IOC_RAMDAC	(KGI_IOCTLBASE+0x02)	/* ramdac driver	*/
#define IOC_CLOCK	(KGI_IOCTLBASE+0x03)	/* clock driver		*/
#define IOC_CHIPSET	(KGI_IOCTLBASE+0x04)	/* chipset driver	*/
#define IOC_ACCEL	(KGI_IOCTLBASE+0x05)	/* accel driver		*/
#define IOC_VIDIO	(KGI_IOCTLBASE+0x06)	/* VideoInOut ports	*/
/*			(KGI_IOCTLBASE+0x07)	   reserved		*/
/*			(KGI_IOCTLBASE+0x08)	   reserved		*/
#define	IOC_MULTI	(KGI_IOCTLBASE+0xff)	/* multi-subsystem-commands */

/* 
 * 	A writing ioctl() sends data to the kernel, a reading gets something
 * 	back. 
 */

#define	IOC(subs,nr)		  _IO(IOC_ ## subs, nr) 
#define	IOR(subs,nr,data)	 _IOR(IOC_ ## subs, nr, data)
#define	IOW(subs,nr,data)	 _IOW(IOC_ ## subs, nr, data)
#define	IRW(subs,nr,data)	_IOWR(IOC_ ## subs, nr, data)

/* 
 *  Probe for KGIcon - this is in FBcon numberspace on purpose. We don't want
 *  to trigger any "panic" reactions on teh fbcon side.
 */
#define KGICON_PROBE		_IOR('F', 0xe8, int)

/* This is the correct return code for a KGI style driver. If you get anything
 * else, abort handling this device as KGI.
 */
#define KGICON_PROBE_MAGIC	0x0049474b

/* 
 * 	driver
 *
 * These are functions that are located inside the "glue" layer that ties 
 * KGI to the native interfaces of the system. They provide basic information
 * about the interface characteristics and have a get/set/testmode interface
 * that is generically available aside from any possible native interfaces.
 */

/* Getting information about the glue layer.
 */
#define	DRIVER_GETINFO		IOR(DRIVER, 0x00, struct kgi_driver)
#define	DRIVER_SETINFO		IOW(DRIVER, 0x00, struct kgi_driver)

#define	DRIVER_STRATEGY		IOC(DRIVER, 0x01)

/* Test function. Returns as soon as possible. Useful for getting 
 * communications channel timing. */
#define	DRIVER_TESTDRIVER	IOC(DRIVER, 0x10)

#define	DRIVER_TESTMODE		IRW(DRIVER, 0x11, kgi_modereq)
#define	DRIVER_SETGRAPHMODE	IRW(DRIVER, 0x12, kgi_modereq)
#define	DRIVER_GETGRAPHMODE	IOR(DRIVER, 0x13, kgi_modereq)

#define	DRIVER_ACCEL		IOC(DRIVER, 0x14)
#define	DRIVER_TEXT		IOC(DRIVER, 0x17)
#define	DRIVER_MMIO_OFFSET	IOC(DRIVER, 0x18)

#define	DRIVER_RAMDAEMON_INSTALL	IOC(DRIVER, 0x20)
#define	DRIVER_RAMDAEMON_ASK	IRW(DRIVER, 0x21, ramdaemon_shm)
#define	DRIVER_RAMDAEMON_CPY	IOW(DRIVER, 0x22, ramdaemon_shm)

/* 
 * 	chipset
 */

#define	CHIP_GETINFO		IOR(CHIPSET, 0x00, struct kgi_chipset)
#define	CHIP_SETINFO		IOW(CHIPSET, 0x00, struct kgi_chipset)

#define	CHIP_SETVISFRAME	IOW(CHIPSET, 0x10, kgi_coord)
#define	CHIP_GETVISFRAME	IOR(CHIPSET, 0x10, kgi_coord)
#define	CHIP_SETSPLITLINE	IOC(CHIPSET, 0x11)
#define	CHIP_GETSPLITLINE	IOC(CHIPSET, 0x12)

#define	CHIP_SETDEFFONT		IOW(CHIPSET, 0x13, struct kgi_font)
#define	CHIP_GETDEFFONT		IRW(CHIPSET, 0x13, struct kgi_font)
#define	CHIP_SETFONTCHAR	IOW(CHIPSET, 0x14, kgi_bm_char)
#define	CHIP_GETFONTCHAR	IRW(CHIPSET, 0x14, kgi_bm_char)

#define CHIP_SETPAGE		IOW(CHIPSET, 0x20, int)
#define CHIP_SET16COL		IOW(CHIPSET, 0x21, int)
#define	CHIP_SETDRAWINFO	IOW(CHIPSET, 0x22, kgi_drawinfo *) 

/* 
 * 	ramdac
 */
struct clut256 { kgi_color clut[256]; };	/* !!! temporary ! */

#define	RAMDAC_GETINFO		IOR(RAMDAC, 0x00, struct kgi_ramdac)
#define	RAMDAC_SETINFO		IOW(RAMDAC, 0x00, struct kgi_ramdac)

#define	RAMDAC_SETCLUT		IOW(RAMDAC, 0x10, struct clut256)	/* FIXME !!! */
#define	RAMDAC_GETCLUT		IRW(RAMDAC, 0x11, struct clut256)	/* FIXME !!! */

/* 
 * 	monitor
 */

#define MONITOR_GETINFO		IOR(MONITOR, 0x00, struct kgi_monitor)
#define MONITOR_SETINFO		IOW(MONITOR, 0x00, struct kgi_monitor)

#define MONITOR_GETCURTIMING	IOR(MONITOR, 0x01, struct kgi_preset)

#define MONITOR_GETNUMPRESET	IOC(MONITOR, 0x10)
#define MONITOR_GETFREEPRESET	IOC(MONITOR, 0x11)
#define MONITOR_SETPRESETMODE	IOW(MONITOR, 0x12, struct kgi_preset)
#define MONITOR_GETPRESETMODE	IRW(MONITOR, 0x12, struct kgi_preset)
#define MONITOR_CLEARPRESETENT	IOW(MONITOR, 0x13, struct kgi_preset)
#define MONITOR_CLEARPRESETS	IOC(MONITOR, 0x13)

#define MONITOR_GETNUMTIMING	IOC(MONITOR, 0x20)
#define MONITOR_GETFREEHTIMING	IOC(MONITOR, 0x21)
#define MONITOR_GETFREEVTIMING	IOC(MONITOR, 0x22)
#define MONITOR_SETHTIMING	IOW(MONITOR, 0x23, struct kgi_timing)
#define MONITOR_GETHTIMING	IRW(MONITOR, 0x23, struct kgi_timing)
#define MONITOR_SETVTIMING	IOW(MONITOR, 0x24, struct kgi_timing)
#define MONITOR_GETVTIMING	IRW(MONITOR, 0x24, struct kgi_timing)
#define MONITOR_CLEARHTIMING	IOW(MONITOR, 0x25, struct kgi_timing)
#define MONITOR_CLEARVTIMING	IOW(MONITOR, 0x26, struct kgi_timing)

#define MONITOR_CLEARTIMINGS	IOC(MONITOR, 0x27)


/* 
 * 	ACCEL
 */

struct kgi_pnt    { int x,y; int col; };
struct kgi_box    { int x,y,w,h; };
struct kgi_fbox   { int x,y,w,h; void *content; };
struct kgi_cbox   { int x,y,w,h,x2,y2; };
struct kgi_line   { int x,y,x2,y2;};
struct kgi_hline  { int x,y,w; };
struct kgi_fhline { int x,y,w; void *content; };
struct kgi_circ   { int x,y,r; };
struct kgi_triangle { int x1,y1,x2,y2,x3,y3; };

typedef struct {
	int fg_color;			/* !!! temporary ! */
	int bg_color;
	int drawmode;
} kgi_drawinfo;

typedef struct kgi_suggest {
	int	id;
	char	name[256];
	char	args[256];
} kgi_suggest;

/* Everything before 0x10 is a `control' function - choose wisely! */
#define ACCEL_GETSUGGEST	IRW(ACCEL, 0x01, kgi_suggest)


/* Everything >= 0x10 is a drawing functions */
#define	ACCEL_DRAWPIXEL		IOW(ACCEL, 0x10, kgi_coord) 
#define	ACCEL_PUTPIXEL		IOW(ACCEL, 0x11, struct kgi_pnt) 
#define	ACCEL_GETPIXEL		IOW(ACCEL, 0x12, struct kgi_pnt) 
#define	ACCEL_DRAWHLINE		IOW(ACCEL, 0x14, struct kgi_hline) 
#define	ACCEL_PUTHLINE		IOW(ACCEL, 0x15, struct kgi_fhline) 
#define	ACCEL_GETHLINE		IOW(ACCEL, 0x16, struct kgi_fhline) 
#define	ACCEL_DRAWLINE		IOW(ACCEL, 0x17, struct kgi_line) 
#define	ACCEL_DRAWVLINE		IOW(ACCEL, 0x18, struct kgi_hline) 
#define	ACCEL_PUTVLINE		IOW(ACCEL, 0x19, struct kgi_fhline) 
#define	ACCEL_GETVLINE		IOW(ACCEL, 0x1a, struct kgi_fhline) 
#define	ACCEL_DRAWBOX		IOW(ACCEL, 0x20, struct kgi_box) 
#define	ACCEL_GETBOX		IOW(ACCEL, 0x21, struct kgi_fbox) 
#define	ACCEL_PUTBOX		IOW(ACCEL, 0x22, struct kgi_fbox) 
#define	ACCEL_FILLSCREEN	IOC(ACCEL, 0x23) 
#define	ACCEL_COPYBOX		IOW(ACCEL, 0x24, struct kgi_cbox) 
#define	ACCEL_DRAWCIRCLE	IOW(ACCEL, 0x30, struct kgi_circ) 
#define	ACCEL_DRAWTRIANGLE	IOW(ACCEL, 0x31, struct kgi_triangle) 

/* VideoIO stuff:
 */

typedef struct          /* requested by user and changed by driver    */
{
	kgiu32          frames;         /* frames needed - double/triplebuffering */
	kgi_coord       visible;        /* vis. pixels */
	kgi_coord       virt;           /* virtual size. Usually used to match stride to an existing mode */
	kgi_coord       offset;		/* offset from centered picture. */
	kgi_coord       scale;		/* Scaling. */
        kgi_graphtype   graphtype;      /* which mode/layout */
        kgi_colormodel	colmodel;	/* Color model. */
} kgi_vidio_reqmode;

typedef int kgi_vidio_avail;	/* Bitmask: 0x1=enable/disable */

#define KGI_VIDIO_AVAIL_ENABLE 1

typedef struct
{
	kgiu32          frame;         /* frame we want to set/get */
	kgiu32          offset;        /* offset from framebuffer start */
} kgi_vidio_framestart;

#define VIDIO_IN_TESTAVAIL	IOR(VIDIO, 0x00, kgi_vidio_avail)
#define VIDIO_IN_ENABLE		IOW(VIDIO, 0x00, kgi_vidio_avail)
#define VIDIO_IN_CHECKMODE	IRW(VIDIO, 0x01, kgi_vidio_reqmode)
#define VIDIO_IN_SETMODE	IRW(VIDIO, 0x02, kgi_vidio_reqmode)
#define VIDIO_IN_GETMODE	IOR(VIDIO, 0x02, kgi_vidio_reqmode)
#define VIDIO_IN_SETFRAMESTART	IOW(VIDIO, 0x03, kgi_vidio_framestart)
#define VIDIO_IN_GETFRAMESTART	IRW(VIDIO, 0x03, kgi_vidio_framestart)

#define VIDIO_OUT_TESTAVAIL	IOR(VIDIO, 0x80, kgi_vidio_avail)
#define VIDIO_OUT_ENABLE	IOW(VIDIO, 0x80, kgi_vidio_avail)
#define VIDIO_OUT_CHECKMODE	IRW(VIDIO, 0x81, kgi_vidio_reqmode)
#define VIDIO_OUT_SETMODE	IRW(VIDIO, 0x82, kgi_vidio_reqmode)
#define VIDIO_OUT_GETMODE	IOR(VIDIO, 0x82, kgi_vidio_reqmode)
#define VIDIO_OUT_SETFRAMESTART	IOW(VIDIO, 0x83, kgi_vidio_framestart)
#define VIDIO_OUT_GETFRAMESTART	IRW(VIDIO, 0x83, kgi_vidio_framestart)

/* 
 * 	multi-commands. These could be supported in various parts
 * 	of the driver. Powersaving is such a thing.
 * 	The kernel driver will ask all subsystems in turn and accept
 *      the first one returning EOK. If none returns EOK, an error
 * 	code is returned.
 */

enum powersave  { PWR_ON=0,PWR_STANDBY,PWR_SUSPEND,PWR_OFF };

#define	MULTI_POWERSAVE		IOW(MULTI, 0x08, enum powersave)

/* 
 * 	The various mmap'ing modes. This is given to the mmap call
 * 	as the offset parameter which is otherwise unused.
 * 	More modes are likely to be added. Especially MMAP_ACCEL.
 */


/* Well known mmap-area types :
*/

#define	MMAP_PER_REGION_TYPE	0x00000000
#define	MMAP_PER_REGION_INDEX	0x80000000

#define	MMAP_ENDIAN		0x40000000
#define	MMAP_LITTLE_ENDIAN	0x00000000
#define	MMAP_BIG_ENDIAN		0x40000000

#define	MMAP_TYPE		0x0F000000
#define	MMAP_TYPE_MMIO		0x00000000
#define	MMAP_TYPE_ACCEL		0x01000000
#define	MMAP_TYPE_GC		0x02000000

#define	MMAP_INDEX		0x0000FFFF
#define	MMAP_PUBLIC		0x00000000
#define	MMAP_PRIVATE		0x00008000
#define	MMAP_FRAMEBUFFER	0x00000001


/* These are for accessing kernel-mode structures
*/
#define MMAP_DRAWINFO		0x0100	/* struct drawinfo writeable by app */

/* 
 * 	ioctl transfer
 * 
 * 	The data transfer is done as follows: 
 * 	(IO_BUF, OUT_BUF and IO_CNT are file local now, may change!)
 * 
 * 	  - The kernel driver checks if the IOC_IN flag in cmd is set. If so,
 * 	    the number of bytes to copy from arg is determined from cmd and
 * 	    copied to _io_buf. _out_buf is set to NULL.
 * 
 * 	  - Then kgi_subsystem_ioctl is invoked according to the subsystem 
 * 	    coded in cmd. This performs the action according to the request in
 * 	    cmd. If output is requested, _out_buf is set to the 'result' buffer
 * 	    and _io_cnt to the number of bytes to copy from there. Note that
 * 	    local variables will be out of scope after return. Use _io_buf then.
 * 
 * 	  - After kgi_subsystem_ioctl has returned, cmd is checked for the 
 * 	    IOC_OUT flag to be set. If so (and _out_buf and _io_cnt are valid),
 * 	    _io_cnt bytes are copied from _out_buf to arg. 
 * 	    
 * 	  - The value returned by kgi_subsystem_ioctl is returned.
 */

#endif	/* #ifdef _KGI_KGI_COMMANDS_H */
